#include "pulsim/circuit.hpp"

// Device-specific implementations will be added here as the project grows.
// For MVP-0, most device behavior is handled in the MNA assembler.

namespace pulsim {

// Placeholder for future device model implementations
// - Advanced MOSFET models
// - IGBT models
// - Magnetic core models
// - etc.

}  // namespace pulsim
